# this code replicates column "BH" of Table 6 of our paper Deckers and Hanck (2012) "Variable Selection in Cross-Section Regressions: Comparisons and Extensions", and generally illustrates
# how to employ Benjamini and Hochberg's (JRSSB 1995) multiple testing procedure for variable selection in linear regressions

# the Fernandez et al. (JAE 2001) data is available in Zeugner's BMS library. 
library(BMS) 
# There are minor rounding differences in this data and the data we work with (supplied in for instance the eviews folder)
# try for instance the first GDP in 1960: 
# datafls[1,"GDP60"]==7.439
# [1] TRUE
#
# but
#
# datafls[1,"GDP60"]==7.438972
# [1] FALSE
#
# this explains minor differences in the higher digits of the p-values, which however do not affect the multiple testing results
data(datafls)
longreg <- lm(datafls) # do the long regression (works as is here because the dependent variable is ordered first, you may need to adjust this with other data!)
longregsumm <- summary(longreg) 
pvals <- longregsumm$coefficients[,4] # get p values 
pvals <- pvals[-1] # without the constant, which is typically ignored in the model selection literature. Does not matter for the present results, though

# Benjamini Hochberg---manually (modified from the appendix to "A Guide to Doing Statistical Analysis in Second Language Research")
sorted.pvalue <- sort(pvals)
gamma_BH = .05  # the chosen gamma
j.alpha <- seq(1:length(pvals))*gamma_BH/length(pvals)
dif <- sorted.pvalue-j.alpha
neg.dif <- dif[dif<0]
pos.dif <- neg.dif[length(neg.dif)]
index <- dif==pos.dif
p.cutoff <- sorted.pvalue[index]
p.sig <- pvals[pvals<=p.cutoff]
p.sig

# given the popularity of multiple testing methods in statistics, many implementations of the Benjamini and Hochberg procedure are available in R.

# some dependencies for mutoss, not all of which are on CRAN and hence not installed automatically
# install.packages("BiocInstaller", repos="http://www.bioconductor.org/packages/2.11/bioc")
# install.packages("Biobase", repos="http://www.bioconductor.org/packages/2.11/bioc")
# install.packages("multtest", repos="http://www.bioconductor.org/packages/2.11/bioc")
library(mutoss)

result <- BH(pvals, gamma_BH)
result$rejected
