mode quiet

'variable to store name of dependent variable
%dep = "growth"
'variable to store list of regressors. in our example for the FLS growth data, this is the constant c and the set of regressors collected in the group allregs. Creating a group is probably more convenient than entering the series names one by one.
'this can certainly be done more elegantly, but we are not seasoned eviews programmers

'note the code assumes that the first regressor is the constant c, which is then later removed in the BH calculations.
'this is to mimic the common practice of the model selection literature to include a constant by default 
%theregs = "c allregs"
'variable that will track the result of dialogs. -1 indicates user hit Cancel. 0 Indicates user hit OK.
!result = 0
!result = @uiedit(%dep,"Benjamini and Hochberg (1995): Enter the dependent variable")
if !result=-1 then 'if user cancelled, then stop program
stop
endif
'put up an Edit dialog asking for regressors list.
!result = @uiedit(%theregs,"Enter the list of regressors. Results are saved in BH_results")
if !result=-1 then 'if user cancelled, then stop program
stop
endif

equation longreg.ls {%dep} {%theregs}
vector(3) gamma
gamma.fill  0.01, 0.05, 0.1
scalar k = longreg.@ncoef-1
vector(k) pvalues 
vector(k) seq
scalar dof = @regobs - k -1
svector(k) names

group regset c allregs

for !i = 1 to k
	seq(!i) = !i
	pvalues(!i) = 2*(1-@ctdist(@abs(longreg.@tstats(!i+1)),dof)) 
	names(!i) = regset.@seriesname(!i+1)
next
matrix(k,3) BH_cutoffs = seq*@transpose(gamma)/k

vector(k) sortedpvalues = @sort(pvalues)
'matrix(k,3) pbelow = @ege(BH_cutoffs,sortedpvalues*@ones(1,3))
'vector jmax_tmp = @cimin(pbelow)-1 ' not robust to double crossing!
'scalar jmax1 = jmax_tmp(1)
'scalar jmax2 = jmax_tmp(2)
'scalar jmax3 = jmax_tmp(3)


!j1=k
while sortedpvalues(!j1)>BH_cutoffs(!j1,1)
	!j1=!j1-1
	scalar jmax1=!j1
wend
!j2=k
while sortedpvalues(!j2)>BH_cutoffs(!j2,2)
	!j2=!j2-1
	scalar jmax2=!j2
wend
!j3=k
while sortedpvalues(!j3)>BH_cutoffs(!j3,3)
	!j3=!j3-1
	scalar jmax3=!j3
wend

matrix(k,3) cutoffmatrix 
colplace(cutoffmatrix,jmax1/k*@ones(k)*gamma(1),1) 
colplace(cutoffmatrix,jmax2/k*@ones(k)*gamma(2),2)
colplace(cutoffmatrix,jmax3/k*@ones(k)*gamma(3),3)

matrix(k,3) rejecindices = @ege(cutoffmatrix,pvalues*@ones(1,3))

table(k+8,6) BH_results
BH_results.setwidth(1) 40
setcell(BH_results,1,1, "Date: " + @datestr(@now,"DD/mm/yyyy") + "  Time: " + @time,"l")
BH_results(2,1)="Decisions for the Benjamini and Hochberg (JRSSB 1995) multiple testing procedure"
BH_results(3,1)="(1 = include the variable at that FDR, 0 = do not include at that FDR)"
setcell(BH_results,5,1, "Dependent variable: " + %dep,"l") 

BH_results(7,2)="p-value"
BH_results(6,5)="Decision at gamma=" 
setcell(BH_results,7,4,@str(gamma(1)),"c")
setcell(BH_results,7,5,@str(gamma(2)),"c")
setcell(BH_results,7,6,@str(gamma(3)),"c")
BH_results.setlines(8) +d

for !i = 1 to k
	BH_results(8+!i,1)=names(!i) 
	BH_results(8+!i,2)=pvalues(!i) 

	setcell(BH_results,8+!i,4,rejecindices(!i,1),-2)
	setcell(BH_results,8+!i,5,rejecindices(!i,2),-2)
	setcell(BH_results,8+!i,6,rejecindices(!i,3),-2)
next

BH_results.setjust(9,A,70,A) left ' does not seem to take more than 70 rows
show BH_results

delete gamma seq BH_cutoffs sortedpvalues jmax1 jmax2 jmax3 k names dof rejecindices pvalues regset cutoffmatrix
